! ----------------------------------------------------------------------
! *   solve linear equations by SOR method for 2-dimensional           *
! *                finite difference method                            *
! ----------------------------------------------------------------------
   subroutine SOR(omeg,eps,nitr,mxm1,mym1)

      use sormod

      implicit none
      integer:: i,j,itr
      integer:: nitr,mxm1,mym1
      real(8)::  omeg,eps
      real(8)::  bnorm,rnorm,gosa,zansa
      real(8),allocatable:: dds(:,:)

      allocate ( dds(mxm1,mym1) )

!  ****    norm of RHS vector for convergent criterion    ****

      bnorm = 0.0d0
      do j = 1, mym1
        do i = 1, mxm1
          bnorm = bnorm + b(i,j)**2
        end do
      end do

!  ****    inverse of coefficient of x(i,j): 1/ap    ****

      do j = 1, mym1
        do i = 1, mxm1
          dds(i,j) = 1.0d0 / ap(i,j)
        end do
      end do

!  ========    iteration    =======================================================

      do itr = 1,nitr

         rnorm = 0.0d0

         do j = 1, mym1
           do i = 1, mxm1
             gosa = b(i,j) - ( as(i,j)*x(i,j-1) + aw(i,j) * x(i-1,j)               &
                              +ae(i,j)*x(i+1,j) + an(i,j) * x(i,j+1)               &
                              +ap(i,j)*x(i,j) )
             rnorm = rnorm + gosa**2
             x(i,j) = x(i,j) + omeg * gosa * dds(i,j)
           end do
         end do

         zansa = sqrt(rnorm/bnorm)
         if ( zansa .lt. eps ) exit

      end do

!  ========    end of iteration    ==============================================

!  ****    end of SOR method    ****

      nitr = itr
      eps  = zansa
      return
   end subroutine sor
