! ----------------------------------------------------------------------
! *     A sample program to solve Poisson equation by SOR method       *
! *                 by nakakuki 20/Dec./1994                           *
! ----------------------------------------------------------------------
   program sormain

      use sormod

      implicit none
      integer:: i, j
      integer:: mxm1, mym1
      integer:: nitr
      real(8),parameter:: ax = 1.0d0, ay = 1.0d0 
      real(8),parameter:: pi = 4.0d0 * atan(1.0d0)
      real(8):: dx, dy, xx, yy
      real(8):: omeg, eps
      character(40):: fmt, ofile

!  ****    set variables  ****

      fmt = '(100e8.2)'
      ofile='sol.dat'

      mxm1 = mx - 1
      mym1 = my - 1
      dx   = ax / dble(mx)
      dy   = ay / dble(mx)

!  ****    set coefficient matrix    ****

      do j = 1, mym1
        do i = 1, mxm1
          as(i,j) =  1.0d0 / dy**2
          aw(i,j) =  1.0d0 / dx**2
          ap(i,j) = -2.0d0 / dx**2 - 2.0d0 / dy**2
          ae(i,j) =  1.0d0 / dx**2
          an(i,j) =  1.0d0 / dy**2
        end do
      end do

!  ****    set boundary condition: xb = 0    ****

      do j = 0,my
        x(0, j) = 0.0d0
        x(mx,j) = 0.0d0
      end do

      do i = 0,mx
        x(i,0)  = 0.0d0
        x(i,my) = 0.0d0
      end do

!  ****    set RHS and initial value    ****

      do j = 1, mym1
        yy = dx * dble(j)
        do i = 1, mxm1
          xx = dx * dble(i)
          b(i,j) = -sin( 2.0d0*pi*xx/ax ) * sin( 3.0d0*pi*yy/ay )
!         write (6,*) b(i,j)
        end do
      end do

      do j = 1, mym1
        do i = 1, mxm1
          x(i,j) = 0.0d0
        end do
      end do

!   ****    solve by SOR method    ****

      omeg = 1.5d0
      nitr = 1000
      eps  = 1.0d-6

      call       SOR(omeg,eps,nitr,mxm1,mym1)

      write (*,*) ' iteration times   = ',nitr
      write (*,*) ' residual norm     = ',eps

!   ****    write unkown x    ****

      do j = 0,my
        write (6,fmt) ( x(i,j), i = 0,mx,1 )
      end do

      open ( 10,file=ofile )
      do j = 0,my
        yy = dx * dble(j)
        do i = 0, mx
          xx = dx * dble(i)
          write (10,*) xx,yy,x(i,j)
        end do
      end do

      stop
   end program sormain

